
#include "readGen.h"

void insertGeneralizedLevelTable(generalizedLevelTable *genLevelTablePtr, int bottomValue, int topLevel, int topValue)
{
	generalizedPair *genPair;
	int noOfPoss;

	// assume it does not exist before

	noOfPoss = genLevelTablePtr->noOfPoss;

	if (genLevelTablePtr->noOfPoss == 0)
	{
		genPair = (generalizedPair *) malloc(sizeof(generalizedPair));
	}
	else
	{
		genPair = (generalizedPair *) realloc(genLevelTablePtr->gen, sizeof(generalizedPair)*(noOfPoss+1));
	}

	genPair[noOfPoss].bottomValue = bottomValue;
	genPair[noOfPoss].topLevel = topLevel;
	genPair[noOfPoss].topValue = topValue;

	genLevelTablePtr->gen = genPair;
	genLevelTablePtr->noOfPoss++;
}

void insertGeneralizedAllTable(generalizedAllTable *genAllTable, configure *conf, int attrNo, int bottomLevel, int bottomValue, int topLevel, int topValue)
{
	generalizedDimTable *genDimTablePtr;
	generalizedLevelTable *genLevelTablePtr;

	if (bottomLevel >= topLevel)
	{
		printf("Bottom level should be smaller than top level!\n");
		printf("Now, the bottom level is %d and the top level is %d for attribute %d.\n", bottomLevel, topLevel, attrNo);
		exit(0);
	}

	if (attrNo >= conf->noOfDim)
	{
		printf("The attribute no. should be smaller than the no. of attributes!\n");
		printf("Now, the attribute no. is %d and the no. of attributes is %d.\n", attrNo, conf->noOfDim);
		exit(0);
	}

	genDimTablePtr = &(genAllTable->genDimTable[attrNo]);

	if (topLevel >= genDimTablePtr->noOfLevel)
	{
		printf("Top level should be smaller than the no. of levels!\n");
		printf("Now, the top level is %d and the no. of levels is %d for attribute %d.\n", topLevel, genDimTablePtr->noOfLevel);
		exit(0);
	}

	genLevelTablePtr = &(genDimTablePtr->gen[bottomLevel]);

	// insert to generalizedLevelTable
	insertGeneralizedLevelTable(genLevelTablePtr, bottomValue, topLevel, topValue);
}

generalizedAllTable *readGen(configure *config)
{
	generalizedAllTable *returnValue;

	int noOfDim;
	int i, j;

	char *filename;

	FILE *filePtr; 

	int noOfRec;
	int attrNo;
	int bottomLevel, bottomValue;
	int topLevel, topValue;

	int noOfLevel;

	generalizedDimTable *genDimTablePtr; 
	generalizedLevelTable *genLevelTablePtr;

	noOfDim = config->noOfDim;
	filename = config->hierarchyFilename;

	filePtr = (FILE *) fopen(filename, "r");
	if (filePtr == NULL)
	{
		printf("File %s does not exist!\n", filename);
		exit(0);
	}

	returnValue = (generalizedAllTable *) malloc(sizeof(generalizedAllTable));
	returnValue->genDimTable = (generalizedDimTable *) malloc(sizeof(generalizedDimTable)*noOfDim);

	for (i = 0; i < noOfDim; i++)
	{
		fscanf(filePtr, "%d\n", &(noOfLevel));

		genDimTablePtr = &(returnValue->genDimTable[i]);

		genDimTablePtr->noOfLevel = noOfLevel;
		genLevelTablePtr = (generalizedLevelTable *) malloc(sizeof(generalizedLevelTable)*noOfLevel);
		genDimTablePtr->gen = genLevelTablePtr;
		for (j = 0; j < noOfLevel; j++)
		{
			genLevelTablePtr[j].noOfPoss = 0;
			genLevelTablePtr[j].gen = NULL;
		}
	}
	fscanf(filePtr, "\n");

	fscanf(filePtr, "%d\n", &noOfRec);

	for (i = 0; i < noOfRec; i++)
	{
		fscanf(filePtr, "%d %d %d %d %d\n", &attrNo, &bottomLevel, &bottomValue, &topLevel, &topValue);
		// insert this to generalizedAllTable returnValue
		insertGeneralizedAllTable(returnValue, config, attrNo, bottomLevel, bottomValue, topLevel, topValue);
	}

	return returnValue;
}
